
#include <uesqlc/defs-uesqlc.cc>
#include "odbc-3.x.hh"

//
//  C O N S T R U C T O R S
//

void uesql::TOdbc3xManager::
connect ( const mpcl::xstring& rkyCONNECTION_NAME ,
          const mpcl::xstring& rkyDATABASE_NAME   ,
          const mpcl::xstring& rkyUSER_LOGIN      )
{

  using mpcl::xstring;
  using odbc_3_x::TConnectionTriple;
  using odbc_3_x::TResultTriple;

  xstring              yUserName;
  xstring              yUserPassword;
  xstring::size_type   zSolidusPos;

  zSolidusPos = rkyUSER_LOGIN.find ('/');
  if ( zSolidusPos == xstring::npos )
  {
    yUserName = rkyUSER_LOGIN;
    yUserName.suppress ('\'');
  }
  else
  {
    yUserName     = rkyUSER_LOGIN.substr (0, zSolidusPos);
    yUserPassword = rkyUSER_LOGIN.substr (zSolidusPos + 1);
    yUserName.suppress ('\'');
    yUserPassword.suppress ('\'');
  }
  if ( isConnected (rkyCONNECTION_NAME) )
  {
    //
    //  Condition:           connection exception
    //  Class subcondition:  connection name in use
    //
    SQLCODE    = -1;
    SQLMESSAGE = "";
    SQLSTATE   = "08002";
  }
  else
  {
    SQLRETURN           tSqlreturn;
    TConnectionTriple   tConnectionTriple (rkyDATABASE_NAME, 0, 0);
    SQLHDBC&        rtSqlhdbc = tConnectionTriple.tSqlhdbc;
    SQLHENV&        rtSqlhenv = tConnectionTriple.tSqlhenv;
    bool                gContinue = true;

    //
    //  1. Allocates environment handle and register version.
    //
    tSqlreturn = SQLAllocHandle (SQL_HANDLE_ENV, SQL_NULL_HANDLE, &rtSqlhenv);
    gContinue  = ( ( tSqlreturn == SQL_SUCCESS ) || ( tSqlreturn == SQL_SUCCESS_WITH_INFO ) );
    if ( !gContinue )
    {
      //
      //  Condition:           connection exception
      //  Class subcondition:  connection failure
      //
      SQLCODE    = -1;
      SQLMESSAGE = "couldn't alloc SQLHENV";
      SQLSTATE   = "08006";
    }
    else
    {
      tSqlreturn = SQLSetEnvAttr (rtSqlhenv, SQL_ATTR_ODBC_VERSION, (void*) SQL_OV_ODBC3, 0);
      gContinue  = ( ( tSqlreturn == SQL_SUCCESS ) || ( tSqlreturn == SQL_SUCCESS_WITH_INFO ) );
      if ( !gContinue )
      {
        //
        //  Condition:           connection exception
        //  Class subcondition:  connection failure
        //
        SQLCODE    = -1;
        SQLMESSAGE = "couldn't register version";
        SQLSTATE   = "08006";
        SQLFreeHandle (SQL_HANDLE_ENV, rtSqlhenv);
      }
    }

    //
    //  2. Allocates connection handle, and sets timeout.
    //
    if ( gContinue )
    {
      tSqlreturn = SQLAllocHandle (SQL_HANDLE_DBC, rtSqlhenv, &rtSqlhdbc);
      gContinue  = ( ( tSqlreturn == SQL_SUCCESS ) || ( tSqlreturn == SQL_SUCCESS_WITH_INFO ) );
      if ( gContinue )
      {
        SQLSetConnectAttr (rtSqlhenv, SQL_LOGIN_TIMEOUT, (SQLPOINTER *) 5, 0);
      }
      else
      {
        //
        //  Condition:           connection exception
        //  Class subcondition:  connection failure
        //
        SQLCODE    = -1;
        SQLMESSAGE = "couldn't allocate SQLHDBC";
        SQLSTATE   = "08006";
        SQLFreeHandle (SQL_HANDLE_ENV, rtSqlhenv);
      }
    }

    //
    //  3. Connect to the data-source.
    //
    if ( gContinue )
    {
      tSqlreturn = SQLConnect ( rtSqlhdbc                           ,
                                (SQLCHAR*) rkyDATABASE_NAME.c_str() ,
                                SQL_NTS                             ,
                                (SQLCHAR*) yUserName.c_str()        ,
                                SQL_NTS                             ,
                                (SQLCHAR*) yUserPassword.c_str()    ,
                                SQL_NTS                             );
      gContinue  = ( ( tSqlreturn == SQL_SUCCESS ) || ( tSqlreturn == SQL_SUCCESS_WITH_INFO ) );
      if ( gContinue )
      {
        TUesqlBaseManager<odbc_3_x::TConnectionTriple, odbc_3_x::TResultTriple>::
          connect (rkyCONNECTION_NAME, tConnectionTriple);
#       ifdef _UESQL_DEBUG__
        mpcl::tLog << mpcl::header << "CONNECT AS '" << rkyCONNECTION_NAME << "';\n";
#       endif  // not _UESQL_DEBUG__
      }
      else
      {
        //
        //  Condition:           connection exception
        //  Class subcondition:  SQL-client unable to establish SQL-connection
        //
        SQLCODE    = -1;
        SQLMESSAGE = "couldn't connect to data source";
        SQLSTATE   = "08001";
        SQLFreeHandle (SQL_HANDLE_ENV, rtSqlhenv);
      }
    }
  }

}  // connect()


void uesql::TOdbc3xManager::
diagnose (uesql::odbc_3_x::TResultTriple tSOURCE_RESULT_TRIPLE)
{

  resetStatus();
  if ( tSOURCE_RESULT_TRIPLE.tSqlreturn == SQL_NO_DATA_FOUND )
  {
    //
    //  Condition:  no data
    // 
    SQLCODE  = 101;
    SQLSTATE = "02000";
  } 
  else if ( ( tSOURCE_RESULT_TRIPLE.tSqlreturn != SQL_SUCCESS )           &&
            ( tSOURCE_RESULT_TRIPLE.tSqlreturn != SQL_SUCCESS_WITH_INFO )  )
  {
    //
    //  Condition:           transaction rollback
    //  Class subcondition:  statement completion unknown
    //
    SQLCODE  = -1;
    SQLSTATE = "40003";
  }

}  // diagnose()


void uesql::TOdbc3xManager::
disconnect (const mpcl::xstring kyCONNECTION_NAME)
{

  using odbc_3_x::TConnectionTriple;
  using odbc_3_x::TResultTriple;

  TConnectionTriple&   rtConnectionTriple = tConnectionMap [kyCONNECTION_NAME];

  SQLDisconnect (rtConnectionTriple.tSqlhdbc);
  SQLFreeHandle (SQL_HANDLE_DBC, rtConnectionTriple.tSqlhdbc);
  SQLFreeHandle (SQL_HANDLE_ENV, rtConnectionTriple.tSqlhenv);
  TUesqlBaseManager<TConnectionTriple, TResultTriple>::
    disconnect (kyCONNECTION_NAME);
# ifdef _UESQL_DEBUG__
  mpcl::tLog << mpcl::header << "DISCONNECT '" << kyCONNECTION_NAME << "';\n";
# endif  // not _UESQL_DEBUG__

}  // disconnect()


uesql::odbc_3_x::TResultTriple uesql::TOdbc3xManager::
execute (const char* pkcSTATEMENT)
{

  uesql::odbc_3_x::TResultTriple   htResultTriple;
  mpcl::xstring                    yStatementInstance (instantiate (pkcSTATEMENT));
  SQLHSTMT&                    rtSqlhstmt  = htResultTriple.tSqlhstmt;
  SQLRETURN&                   rtSqlreturn = htResultTriple.tSqlreturn;

# ifdef _UESQL_DEBUG__
  mpcl::tLog << mpcl::header << yStatementInstance << std::endl;
# endif  // not _UESQL_DEBUG__
  rtSqlreturn = SQLAllocHandle (SQL_HANDLE_STMT, tCurrentConnection.tSqlhdbc, &rtSqlhstmt);
  if ( ( rtSqlreturn == SQL_SUCCESS ) || ( rtSqlreturn == SQL_SUCCESS_WITH_INFO ) )
  {
    rtSqlreturn = SQLExecDirect (rtSqlhstmt, (SQLCHAR*) yStatementInstance.c_str(), SQL_NTS);
  }
  return htResultTriple;

}  // execute()


uesql::odbc_3_x::TResultTriple uesql::TOdbc3xManager::
execute ( const char* pkcSTATEMENT       ,
          const char* pkcCONNECTION_NAME )
{

  uesql::odbc_3_x::TResultTriple   htResultTriple;
  SQLHSTMT&                    rtSqlhstmt  = htResultTriple.tSqlhstmt;
  SQLRETURN&                   rtSqlreturn = htResultTriple.tSqlreturn;

  if ( !tConnectionMap.is_bound (pkcCONNECTION_NAME) )
  {
    //
    //  Condition:           connection exception
    //  Class subcondition:  connection does not exist
    //
    SQLCODE    = -1;
    SQLMESSAGE = "";
    SQLSTATE   = "08003";
  }
  else
  {
    mpcl::xstring   yStatementInstance (instantiate (pkcSTATEMENT));

#   ifdef _UESQL_DEBUG__
    mpcl::tLog << mpcl::header << yStatementInstance << std::endl;
#   endif  // not _UESQL_DEBUG__
    rtSqlreturn = SQLAllocHandle ( SQL_HANDLE_STMT                              ,
                                   tConnectionMap [pkcCONNECTION_NAME].tSqlhdbc ,
                                   &rtSqlhstmt                              );
    if ( ( rtSqlreturn == SQL_SUCCESS ) || ( rtSqlreturn == SQL_SUCCESS_WITH_INFO ) )
    {
      rtSqlreturn = SQLExecDirect (rtSqlhstmt, (SQLCHAR*) yStatementInstance.c_str(), SQL_NTS);
    }
  }
  return htResultTriple;

}  // execute()


void uesql::TOdbc3xManager::finish (void)
{

  TStringToResultTriple::iterator   I = tCursorMap.begin();

  for (; ( I != tCursorMap.end() ) ;I++)
  {
    clearResult (I->second);
  }
  TUesqlBaseManager<odbc_3_x::TConnectionTriple, odbc_3_x::TResultTriple>::
    finish();

}  // finish()


void uesql::TOdbc3xManager::initialize (void)
{

  connect ("DEFAULT", "", "");
  if ( !isSqlerrorCondition() )
  {
    setCurrentConnection ("DEFAULT");
  }

}  // initialize()


void uesql::TOdbc3xManager::updateOutputVariables (void)
{

  std::vector<TSqlDataType*>::iterator         I1;
  std::vector<SQLINTEGER>::iterator            I2;
  std::vector<TSqlDataType*>::const_iterator   tEndIter1;

  I1        = tOutputEmbeddedVariableVector.begin();
  I2        = tOutputEmbeddedVariableLengthVector.begin();
  tEndIter1 = tOutputEmbeddedVariableVector.end();
  for (; ( I1 != tEndIter1 ) ;I1++, I2++)
  {
    if ( *I2 == SQL_NULL_DATA )
    {
      (*I1)->set (NULL);
    }
    else
    {
      (*I1)->updateFromBuffer();
    }
  }
# ifdef _UESQL_DEBUG__
  mpcl::tLog << mpcl::header << "FETCHED (";
  if ( !tOutputEmbeddedVariableVector.empty() )
  {
    mpcl::tLog << tOutputEmbeddedVariableVector.front()->get();
    std::for_each ( tOutputEmbeddedVariableVector.begin() + 1 ,
                    tOutputEmbeddedVariableVector.end()       ,
                    MWriteDebugInfo<TSqlDataType>()     );
  }
  mpcl::tLog << ");\n";
# endif  // not _UESQL_DEBUG__

}  // updateOutputVariables()


//
//  E X P O R T E D   V A R I A B L E S
//

uesql::TOdbc3xManager   uesql::htOdbc3xManager;
